#include <stdint.h>
#include <multiboot.h>
#include <cpu.h>
#include <memory.h>
#include <terminal.h>
#include <debug.h>
#include <proc.h>
#include <musl-glue.h>

#include <ports.h>

void TEMP_test_scheduler();

struct kernel_boot_data_st kernel_boot_data;

registers *kbd_handler(registers *r)
{
  while(inb(0x64) & 0x2);
  uint8_t scancode = inb(0x60);
  debug("Keyboard: %x\n", scancode);
  irq_ack();
  return r;
}

void kmain(uint64_t multiboot_magic, void *multiboot_data) {
  musl_init();
  debug_info("Started kernel\n");
  multiboot_init(multiboot_magic, P2V(multiboot_data));
  debug_info("Kernel loaded with command line: \"%s\" by <%s>\n",
      kernel_boot_data.commandline,
      kernel_boot_data.bootloader
  );

  early_cpu_init();
  debug_info("Set up boot CPU\n");
  memory_init();
  debug_info("Set up memory management\n");

  terminal_init();
  debug_info("Set up debug terminal\n");

  cpu_init();

  __asm__("sti");

  debug_info("Boot complete\n");


  bind_interrupt(IRQ_INTERRUPT(IRQ_PS2_KBD), kbd_handler);
  irq_unmask(IRQ_PS2_KBD);
  TEMP_test_scheduler();


  start_scheduler();

  PANIC("End of kernel function!");

  debug_info("Broke out of panic");
  for(;;);
}