#pragma once
#include <stdint.h>
#include <memory.h>

#define MSR_APIC_BASE 0x0000001B

#define MAX_CPUS 16
#define MAX_IRQS 24

#define IRQ_BASE 0x20
#define IRQ_INTERRUPT(irq) (IRQ_BASE + (irq))

#define IRQ_PS2_KBD 0x1
#define IRQ_TIMER 0x20
#define IRQ_SPURIOUS 0xFF

#define APIC_BASE 0xFEE00000
#define APIC_REG(r) (((uint32_t *)P2V(APIC_BASE + (r)))[0])
#define APIC_EOI 0xB0
#define APIC_SPURIOUS 0xF0
#define APIC_LVT(r) (0x320 + 0x10*(r))
#define LVT_TIMER 0


typedef struct {
  uint64_t rax;
  uint64_t rbx;
  uint64_t rcx;
  uint64_t rdx;
  uint64_t rsi;
  uint64_t rdi;
  uint64_t rbp;
  uint64_t r8;
  uint64_t r9;
  uint64_t r10;
  uint64_t r11;
  uint64_t r12;
  uint64_t r13;
  uint64_t r14;
  uint64_t r15;

  uint64_t int_no;
  uint64_t err_code;

  uint64_t rip;
  uint64_t cs;
  uint64_t rflags;
  uint64_t rsp;
  uint64_t ss;
} registers;

typedef registers *(*int_handler_t)(registers *);

struct cpu {
  uint8_t id;
  uint8_t apic_id;
  uint32_t flags;
};
struct ioapic {
  uint8_t id;
  uint32_t addr;
  uint32_t base;
};

extern struct cpu *cpus[MAX_CPUS];
extern struct ioapic ioapic;
extern uint8_t irq_redirects[MAX_IRQS];

// cpu/cpu.c
void early_cpu_init();
void cpu_init();

// cpu/registers.S
void load_idt(void *);
void write_cr3(uint64_t);
uint64_t read_msr(uint32_t);
uint64_t write_msr(uint32_t msr, uint64_t value);

// cpu/interrupts.c
void interrupt_init();
void isr_return(registers *);
int_handler_t bind_interrupt(uint32_t num, int_handler_t fn);

// cpu/acpi.c
void acpi_parse(void *_rsdp);

// cpu/apic.c
void apic_init();

// cpu/ioapic.c
void irq_ack();
void irq_mask(int irq);
void irq_unmask(int irq);
void ioapic_init();

// cpu/timer.c
void timer_init();